﻿#include "syscontactmgt.h"
#include "syscontactmgtthread.h"
#include <QDockWidget>
#include <QFileDialog>
#include <QGraphicsDropShadowEffect>
#include <QToolBar>
#include <QVBoxLayout>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/tfileio.h>
#include <tbaseutil/tlogger.h>
#include <tbaseutil/tresource.h>
#include <tbaseutil/ttheme.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topcore.h>
#include <topcore/topenummanager.h>
#include <toputil/t.h>
#include <twidget/tadvancedquery.h>
#include <twidget/taccordion.h>
#include <twidget/tcategorytreeview.h>
#include <twidget/tmessagebar.h>
#include <twidget/tmessagebox.h>
#include <twidget/tpagetool.h>
#include <twidget/tscrollarea.h>
#include <twidget/tsearchentry.h>
#include <twidget/tsplitter.h>
#include <twidget/ttablechooserdialog.h>
#include <twidget/ttableviewdialog.h>
#include <twidget/ttableview.h>
#include <twidget/ttabwidget.h>
#include <twidget/tuiloaderdialog.h>


#define RELATEION_DATA_KEY "RelationDataKey"

SysContactMgt::SysContactMgt(const QString &iModuleNameStr, const QVariantMap iUrlPars, QWidget *iParent) :
    TopClassAbs(iParent)
{
    this->appendLanguage("sys-contact");
    this->setLicenseKey("");
    this->initModule(iModuleNameStr, iUrlPars);

    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }

    mBodySplitter = new TSplitter(this);
    this->setCentralWidget(mBodySplitter);

    //初始化TableView
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
    initTableView();

    mBodyWidget = new TSplitter(this);
    mBodySplitter->addWidget(mBodyWidget);

    QFrame *centerWgt = new QFrame(this);
    QVBoxLayout* vlayout = new QVBoxLayout(centerWgt);
    vlayout->setMargin(0);
    vlayout->setSpacing(0);

    //工具栏
    QString toolBarStr;
    QString menuBarStr;
    if (config("relationTable").toString().isEmpty()) {
        toolBarStr = "DEFAULT_MAIN_TOOLBAR";
        menuBarStr = "DEFAULT_TABLEVIEW_POPUP";
        mDetailView = new SysContact("sys-contact", QVariantMap(), this);
        connect(mDetailView, SIGNAL(dataSaved(QVariant)), this, SLOT(onDetailSaved(QVariant)));
        connect(mDetailView, SIGNAL(windowModifyChanged(bool)), SLOT(onDetailChanged()));
        mBodySplitter->addWidget(mDetailView);
        initNaviView();
    } else {
        toolBarStr = "MAIN_TOOLBAR";
        menuBarStr = "TABLEVIEW_POPUP";
    }

    if (QMenu *table_popup = qobject_cast<QMenu *>(uim()->getWidget(menuBarStr))) {
        mTableView->setContextMenu(table_popup);
    }

    mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget(QString("%1/SEARCH_ENTRY").arg(toolBarStr)));

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget(toolBarStr))) {
        toolbar->setWindowTitle(ttr("ToolBar"));
        vlayout->addWidget(toolbar, 0);
    }
    vlayout->addWidget(mTableView,0);

    //翻页工具
    mPageTool = new TPageTool(this);
    vlayout->addWidget(mPageTool);

    mBodyWidget->addWidget(centerWgt);

    mBodySplitter->setStretchFactor(0, 1);
    mBodyWidget->setStretchFactor(1, 1);

    mBodyWidget->setSizes(QList<int>() << TTHEME_DP(config("navi.perfect_size.width", 150).toInt()) << 1);
    mBodySplitter->setSizes(QList<int>() << 1 << TTHEME_DP(config("detail.perfect_size.width", 600).toInt()));

    if (mSearchEntry != nullptr) {
        mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
        mSearchEntry->setPlaceholderText(ttr("Search %1"));
        connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
        this->setFocusProxy(mSearchEntry);
    }

    if (mPageTool != nullptr) {
        mPageTool->setPageSizeVisible(true);
        connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
    }

    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));

    if (QAction *openAct = this->getAction("open")) {
        connect(mTableView, SIGNAL(activated(QModelIndex)), openAct, SLOT(trigger()));
    }

    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }

    this->restoreSizeState();
    this->restoreObjectState(mTableView);
    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refresh()));
}

SysContactMgt::~SysContactMgt()
{
    this->saveSizeState();
    this->saveObjectState(mTableView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

void SysContactMgt::refresh(bool iResetPageBol)
{
    t::loading(this);
    TSqlSelectorV2 selector = getSqlSelector(iResetPageBol);
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    unloading();
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
    } else {
        if (mPageTool != nullptr) {
            mPageTool->setRowCount(dataRes.dataCount(), true);
        }
        QVariantList dataList = dataRes.data().toList();
        TopClassHelper::formatTableData(this, &mTableConf, dataList);
        if (isHookExists("handleTableData")) {
            dataList = callHooksQuick("handleTableData", QVariantList() << QVariant(dataList)).toVariant().toList();
        }
        mTableView->loadData(dataList);
        alertOk(ttr("Data loaded"));
    }
}

void SysContactMgt::deleteItem(const QVariantList &iIdLst)
{
    loading(ttr("Deleting data..."));
    QVariant data = doThreadWork(new SysontactMgtThread(this), "DELETE_USER", iIdLst);
    unloading();
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Delete data failed!"), dataRes.errText());
    } else {
        refresh(true);
        alertOk(ttr("Data deleted"));
    }
}

QVariantList SysContactMgt::selectedItems()
{
    return mSelectedLst;
}

TopClassAbs *SysContactMgt::openItem(int iUserId)
{
    QString detailModuleName = "sys-contact";
    if (!config("detailModuleName").toString().isEmpty()) {
        detailModuleName = config("detailModuleName").toString();
    }
    SysContact *detail = qobject_cast<SysContact *>(APP->openModule(detailModuleName));
    if (detail != nullptr) {
        connect(detail, SIGNAL(dataSaved(QVariant)), this, SLOT(onDetailSaved(QVariant)));
        connect(this, SIGNAL(destroyed()), detail, SLOT(close()));
        detail->setUid(iUserId);
    }
    return detail;
}

void SysContactMgt::newItem()
{
    mBodySplitter->cacheSizes();
    mDetailView->create();
    mDetailView->refreshActionState();
}

void SysContactMgt::addContacts()
{
    TTableViewDialog *dialog = new TTableViewDialog(this);
    dialog->resize(800,400);
    //dialog->setIcon("");
    dialog->setTitle( ttr("Select Contacts"));
    dialog->setButtons(QStringList() << "Ok" << "Cancel");

    QVariantList headeritems;
    headeritems << QVariant();
    QVariantMap col1;
    col1.insert("name","id");
    col1.insert("display",ttr("Id"));
    col1.insert("resizeMode","Interactive");
    col1.insert("size",100);
    col1.insert("displayRole","$id");
    headeritems.append(col1);

    QVariantMap col2;
    col2.insert("name","name");
    col2.insert("display",ttr("Name"));
    col2.insert("resizeMode","Interactive");
    col2.insert("size",100);
    col2.insert("displayRole","$name");
    headeritems.append(col2);

    QVariantMap col3;
    col3.insert("name","sex");
    col3.insert("display",ttr("Sex"));
    col3.insert("resizeMode","Interactive");
    col3.insert("size",100);
    col3.insert("displayRole","$sex");
    headeritems.append(col3);

    QVariantMap col0;
    col0.insert("name","status");
    col0.insert("display",ttr("Status"));
    col0.insert("resizeMode","Interactive");
    col0.insert("size",100);
    col0.insert("displayRole","$status");
    headeritems.append(col0);

    QVariantMap col4;
    col4.insert("name","source");
    col4.insert("display",ttr("Source"));
    col4.insert("resizeMode","Interactive");
    col4.insert("size",100);
    col4.insert("displayRole","$source");
    headeritems.append(col4);

    QVariantMap col5;
    col5.insert("name","source");
    col5.insert("display",ttr("Source Serial"));
    col5.insert("resizeMode","Interactive");
    col5.insert("size",100);
    col5.insert("displayRole","$source");
    headeritems.append(col5);

    QVariantMap col6;
    col6.insert("name","company");
    col6.insert("display",ttr("Company"));
    col6.insert("resizeMode","Interactive");
    col6.insert("size",150);
    col6.insert("displayRole","$company");
    headeritems.append(col6);

    QVariantMap col7;
    col7.insert("name","department");
    col7.insert("display",ttr("Department"));
    col7.insert("resizeMode","Interactive");
    col7.insert("size",150);
    col7.insert("displayRole","$department");
    headeritems.append(col7);

    QVariantMap col8;
    col8.insert("name","title");
    col8.insert("display",ttr("Title"));
    col8.insert("resizeMode","Interactive");
    col8.insert("size",120);
    col8.insert("displayRole","$title");
    headeritems.append(col8);

    QVariantMap col9;
    col9.insert("name","remark");
    col9.insert("display",ttr("Remark"));
    col9.insert("resizeMode","Stretch");
    col9.insert("displayRole","$remark");
    headeritems.append(col9);

    QStringList dbDataLst;
    dbDataLst << "id" << "name" << "sex" << "status" << "source" << "source_serial" << "company"
               << "department" << "title" << "remark" << "sex.text" << "sex.icon" ;
    dialog->setPrimaryKey("name");
    dialog->setDataKeyList(dbDataLst);
    dialog->setHeaderItem(headeritems);
    dialog->searchEntry()->setPlaceholderText(ttr("Search"));
    dialog->searchEntry()->setSearchDelayTime(1);
    dialog->setSearchKeys(QStringList() << "name" );
    dialog->tableView()->verticalHeader()->setVisible(false);
    dialog->setSearchText("");
    dialog->setSelectionMode(QAbstractItemView::ExtendedSelection);

    QVariantList existIdLst = mTableView->allPrimaryKey();
    QVariantList allDbData = getAllContactList(existIdLst);
    TopClassHelper::formatTableData(this, &mTableConf, allDbData);
    dialog->loadData(allDbData);

    QVariantList selList = dialog->run();
    if (selList.isEmpty()) {
        return;
    }

    saveContactListInfo(selList);
    return ;
}

bool SysContactMgt::removeItem(int iItemId)
{
    loading(ttr("Deleting data..."));
    QVariantMap relationDataMap = getRelationData();
    TSqlDeleterV2 selector;
    selector.setTable(config("relationTable").toString());
    selector.setWhere(relationDataMap.value("relationTableIdName").toString(),config("defaultData"));
    selector.addWhere(relationDataMap.value("relationTableContactName").toString(),QString::number(iItemId));

    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_DELETE_ROW, QVariant::fromValue(selector));
    unloading();
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Delete data failed!"), dataRes.errText());
        return false;
    } else {
        alertOk(ttr("Delete data success!"));
    }
    return true;
}

bool SysContactMgt::canModify()
{
    return !mDetailView->isDataModified();
}

void SysContactMgt::onPageChanged()
{
    refresh(false);
}

void SysContactMgt::onSelectionChanged()
{
    if (mTableView != nullptr) {
        //因为Action的State函数中用到getSelectedList, 所以TableView选择更改时,
        //将其保存至mSelectedLst中, 这样可以加快Action状态的刷新速度;
        mSelectedLst = mTableView->selectedPrimaryKeys();

        int curIdInt = 0;
        if (!mSelectedLst.isEmpty()) {
            curIdInt = mSelectedLst.value(0).toInt();
        }
        if(mDetailView != nullptr) {
            if (mDetailView->uid().toInt() != curIdInt) {
                //设置详情模块的上次Uid为当前选择的ID, 这样当如果详情模块处于编辑状态时点取消后显示的当前择的条目内容;
                mDetailView->setLastUid(curIdInt);
                if (!mDetailView->isDataModified()) {
                    mDetailView->setUid(curIdInt);
                }
            }
        }
        refreshActionState();
    }
}

void SysContactMgt::onDetailSaved(const QVariant &iUidVar)
{
    this->refresh(false);
    mTableView->selectRow(iUidVar);
}

void SysContactMgt::onCategoryViewDataChanged(QVariantList iDataLst)
{
    mCategoryWhere.clear();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (!iDataLst.isEmpty()) {
        if (format.compare("filter_by_enum") == 0) {
            QVariantMap cateMap;
            for (QVariant item: iDataLst) {
                QVariantMap itemMap = item.toMap();
                QString data = itemMap.value("data").toString();
                QString name = itemMap.value("name").toString();
                if (data.isEmpty()) {
                    continue;
                }
                cateMap.insertMulti(data,name);
            }
            for (QString field: cateMap.uniqueKeys()) {
                QString dbField = field;
                TSqlWhereCompsiteV2 childWhere;
                childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
                for (QVariant ele: cateMap.values(field)) {
                    QString value = ele.toString();
                    if (mTableConf.fieldFormatMap.contains(dbField)) {
                        QString fieldValue = mTableConf.fieldFormatMap.value(dbField).toString();
                        if (fieldValue.compare("array") == 0) {
                            QString sqlStr = QString("%1 @> '{%2}' ").arg(dbField).arg(value);
                            childWhere.append(sqlStr);
                        }
                    } else {
                        childWhere.append(dbField, value);
                    }
                }
                mCategoryWhere.append(childWhere);
            }
        } else if (format.compare("filter_by_sql_where") == 0) {
            QVariantMap item = iDataLst.at(0).toMap();
            QString sqlWhere = item.value("data").toString();
            if (!sqlWhere.isEmpty()) {
                mCategoryWhere.append(sqlWhere);
            }
        }
    }
    refresh();
}

void SysContactMgt::onDetailChanged()
{   
    bool isModified = mDetailView->isDataModified();
    mTableView->setEnabled(!isModified);
    if (isModified) {
        this->showMaskFrame(mBodyWidget);
    } else {
        if (mBodySplitter->sizes().value(0) < 20) {
            mBodySplitter->setWidgetSize(0, mBodySplitter->getCacheSize(0), true);
        }
        this->hideMaskFrame();
    }
    refreshActionState();
}
/**
* \brief 当高级查询界面的添加按钮触发时的槽函数
*/
void SysContactMgt::onAddActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableChooserDialog *dialog = new TTableChooserDialog(this);
    dialog->setTitle(tr("Please select field:"));
    dialog->setButtons(QStringList()
                       << ttr("Ok") + ":Ok:Yes"
                       << ttr("Cancel") + ":Cancel:Cancel"
                       << QString("%1:UnselectAll:ResetRole").arg(tr("Clear selection")));
    QVariantList headeritems;
    headeritems << QVariant() << TDataParse::stringList2VariantMap(
                       QStringList() << "name" << "title" << "display" << tr("Field") << "displayRole" << "$title"<<"resizeMode" << "Stretch");
    dialog->setHeaderItem(headeritems);
    dialog->setDataKeyList(QStringList() << "title" << "name");
    dialog->setPrimaryKey("name");
    dialog->setSearchKeys(QStringList() << "title");
    dialog->setLeftTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->setRightTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->tableView()->verticalHeader()->setVisible(false);
    dialog->selectedTableView()->verticalHeader()->setVisible(false);
    dialog->loadAllData(getAdvancedQueryFieldList());
    dialog->loadSelectedData(advancedQuery->getQueryData().value("value").toList());

    QVariantList ret = dialog->run();
    if (ret.isEmpty()) {
        return;
    }

    QStringList conditionLst;
    for (QVariant var : ret) {
        QString fieldName = var.toMap().value("name").toString();
        conditionLst << fieldName;
    }
    advancedQuery->clearSearchCondition();
    advancedQuery->addSearchCondition(conditionLst);
}

/**
* \brief 当高级查询界面的清除按钮触发时的槽函数
*/
void SysContactMgt::onClearActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }
    advancedQuery->clearSearchCondition();
}

/**
* \brief 当高级查询界面的另存为按钮触发时的槽函数
*/
void SysContactMgt::onSaveAsActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getSaveFileName(this,
                                                    tr("Save query criteria as file"),
                                                    QString(),
                                                    QString("*.cond"));
    if (fileName.isEmpty()) {
        return;
    }

    QVariantMap newConditionMap;
    newConditionMap.insert("condition", advancedQuery->getQueryData());
    QString conditionStr = TDataParse::variant2JsonStr(newConditionMap);
    TFileIo::writeFileString(conditionStr , fileName);
}

/**
* \brief 当高级查询界面的打开按钮触发时的槽函数
*/
void SysContactMgt::onOpenActtionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Open query criteria from file"),
                                                    ".",
                                                    tr("Condition file(*.cond)"));
    if (fileName.isEmpty()) {
        return;
    }
    QString conditionStr = TFileIo::readFileString(fileName);
    QVariantMap dataMap = TDataParse::jsonStr2Variant(conditionStr).toMap();
    QVariantMap conditionMap = dataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
}

/**
* \brief 当高级查询界面的保存按钮触发时的槽函数
*/
void SysContactMgt::onSaveAsCommonQueryActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QString uiStr;
    uiStr = "function func(self) {"
                   " return [{"
                           " name: 'basic_formlayout',"
                            "type: 'FormLayout',"
                            "property:{"
                                    "label_alignment: 'Top | Right',"
                                    "horizontal_spacing: 20, vertical_spacing: 10, margin: 20"
                            "},"
                            "child: [{"
                                    "name: 'name',"
                                    "type: 'LineEdit',"
                                    "title: '" + tr("Query Name") + "',"
                                    "pack: {"
                                            "label: '" + tr("Query Name") + "'"
                                    "},"
                                    "validate: 'NOTNULL'"
                                    "}]"
                            "}]}";

    TUiloaderDialog *dialog = new TUiloaderDialog(this);
    dialog->setTitle(tr("Save as a common query"));
    dialog->setSelf(this);
    dialog->resize(250,100);
    dialog->setButtons(QStringList() << QString("%1:Ok:AcceptRole").arg(ttr("Ok")) << QString("%1:Cancel::RejectRole").arg(ttr("Cancel")));
    dialog->setUiStr(uiStr);

    QVariantMap ret = dialog->run(true);
    if (ret.isEmpty()) {
        return;
    }

    // 判断name是否已经存在
    QString conditionName = ret.value("name").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();
    for (QVariant var: conditionList) {
        QVariantMap itemMap = var.toMap();
        if (conditionName == itemMap.value("title")) {
            TMessageBox::error(this, tr("Save as common condition"), tr("This name already exists!"));
            return;
        }
    }

    // 取新的查询条件
    QVariantMap conditionMap = advancedQuery->getQueryData();
    QVariantMap newConditionMap;
    newConditionMap.insert("title", conditionName);
    newConditionMap.insert("condition", conditionMap);
    conditionList << QVariant(newConditionMap);
    mAdvancedQueryTableDataMap.insert("advance", conditionList);

    // 生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    saveMap.insert("version", 1); // 因为sys_conf的version字段不能为空，因此这里设为1
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";

    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    } else {
        commonQueryTable->loadData(conditionList);
    }
}

/**
* \brief 当常用查询界面的表格双击条目时的槽函数
*/
void SysContactMgt::onCommonQueryTableDoubleClicked(const QModelIndex &index)
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    TTabWidget *advancedQueryTabWidget = findAdvancedQueryTabWidget();
    if (advancedQueryTabWidget == nullptr) {
        return;
    }

    advancedQuery->clearSearchCondition();
    TTableModel *tableModel = qobject_cast<TTableModel *>(commonQueryTable->model());
    QVariantMap rowDataMap = tableModel->rowDataMap(index);
    QVariantMap conditionMap = rowDataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
    advancedQueryTabWidget->setCurrentIndex(0);
}

/**
* \brief 当常用查询界面的表格删除条目时的槽函数
*/
void SysContactMgt::onDeleleCommonQueryActtionTriggered()
{
    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QVariantMap selectedConditionMap = commonQueryTable->currentRowDataMap().toMap();
    if (selectedConditionMap.isEmpty()) {
        return;
    }

    QString selectedTitle = selectedConditionMap.value("title").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();;
    QVariantList conditionDataList;
    for (QVariant var : conditionList){
        QString title = var.toMap().value("title").toString();
        // 如果不是选中的项，表示需要保留
        if (title != selectedTitle) {
            conditionDataList << QVariant(var.toMap());
        }
    }
    commonQueryTable->loadData(conditionDataList);
    mAdvancedQueryTableDataMap.insert("advance", conditionDataList);

    //生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    }
}

void SysContactMgt::onNaviReset()
{
    TUiLoader *filterUi = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        filterUi = mNaviViewUi;
    } else {
        filterUi = qobject_cast<TUiLoader*>(mNaviView);
    }
    if (filterUi == nullptr) {
        return;
    }
    filterUi->clearValues();
    this->refresh();
}

TSqlSelectorV2 SysContactMgt::getSqlSelector(bool iResetPage)
{
    TSqlSelectorV2 selector;
    selector.setTable(mTableConf.dbTableName);
    selector.setField(mTableConf.queryFields);
    selector.setFieldFormat(mTableConf.fieldFormatMap);
    selector.setReturnRowCount(true);
    // 搜索分页
    TopClassHelper::handleSearchPageOnQuery(mSearchEntry, mPageTool, iResetPage, &selector);
    // 排序
    TopClassHelper::handleOrderOnQuery(mTableView, &mTableConf, &selector);
    // 只获取有效数据
    if (!mTableConf.dbDelFlagKey.isEmpty()) {
        selector.addWhere(QString("%1 = 0 OR %1 IS NULL").arg(mTableConf.dbDelFlagKey));
    }
    // 过滤项
    if(!mTableConf.dbFilter.isEmpty()) {
        selector.addWhere(mTableConf.dbFilter);
    }
    // 导航栏过滤项
    if (mNaviType == NaviType::None) {
        // do nothing
    } else if (mNaviType == NaviType::UiLoader) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            selector.addWhere(whereStr);
        }
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
        TAdvancedQuery *advancedQuery = findAdvancedQuery();
        if (advancedQuery != nullptr) {
            if (!advancedQuery->getSql().isEmpty()) {
                selector.addWhere(advancedQuery->getSql());
            }
        }
    } else if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            selector.addWhere(whereStr);
        }
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
    } else {
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
    }

    if (!config("relationTable").toString().isEmpty() && !config("defaultData").toString().isEmpty() ) {
        QStringList customerIds = listRelationCustomerId();
        if (customerIds.isEmpty()) {
            selector.whereRef().append("id = 0");
        } else {
            selector.whereRef().append("id IN (" + customerIds.join(",") + ")");
        }
    }
    return selector;
}

void SysContactMgt::initNaviView()
{
    mNaviType = str2NaviType(config("navi.__type__", "CategoryTreeView").toString());
    if (mNaviType == NaviType::None) {
        return;
    }

    TAccordion *accordion = new TAccordion(this);
    // 此ObjectName在其他逻辑中是有意义的，请勿删除
    accordion->setObjectName("__navi__");
    accordion->setProperty("SS_BG","NAVI");
    accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
    accordion->setMaximumWidth(TTHEME_DP(config("navi.max_width", 300).toInt()));
    accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
    QVBoxLayout *naviLayout = new QVBoxLayout(accordion);
    naviLayout->setMargin(0);
    naviLayout->setSpacing(0);
    QLabel *naviLabel = new QLabel(tr("Navigation"));
    naviLabel->setProperty("SS_TYPE", "SUBHEADER");
    naviLayout->addWidget(naviLabel,0);
    naviLayout->addSpacing(TTHEME_DP(4));
    accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
    accordion->expandButton()->setToolTip(tr("Show Navigation"));
    accordion->collapseButton()->setToolTip(tr("Hide Navigation"));
    accordion->setIsExpanded(config("navi.is_expanded", false).toBool());
    mBodyWidget->insertWidget(0, accordion);

    if (mNaviType == NaviType::UiLoader) {
        mNaviView = new TUiLoader(this);
        TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView);
        filterUi->setSelf(this);
        filterUi->setScriptEngine(APP->scriptEngine());
        filterUi->setUiStr(ui("navi").toString());

        QFrame *naviWgt = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(naviWgt);
        leftLayout->addWidget(mNaviView);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftBtnsLayout->setMargin(0);
        leftBtnsLayout->setSpacing(10);
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), this, SLOT(onNaviReset()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refresh()));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        leftLayout->addStretch(1);
        naviLayout->addWidget(naviWgt);
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        // tab1-CategoryTreeView
        TCategoryTreeView *categoryTreeView = new TCategoryTreeView(this);
        categoryTreeView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));

        // tab2-tab1-ToolBar
        QToolBar *toolbar = new QToolBar(this);
        toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        toolbar->setProperty("SS_SIZE", "SMALL");
        toolbar->setMovable(false);

        QAction *queryAction = toolbar->addAction(APP->getResIcon("search"), tr("Query"));
        connect(queryAction, SIGNAL(triggered()), this, SLOT(refresh()));
        toolbar->addSeparator();

        QAction* addAction = toolbar->addAction(APP->getResIcon("addto-circle-o"), tr("Add"));
        addAction->setToolTip(tr("Add query criteria"));
        connect(addAction, SIGNAL(triggered()), this, SLOT(onAddActionTriggered()));

        QAction* clearAction = toolbar->addAction(APP->getResIcon("eraser"), tr("Clear"));
        clearAction->setToolTip(tr("Clear all query criteria"));
        connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionTriggered()));

        QAction* saveAsAction = toolbar->addAction(APP->getResIcon("save"), tr("Save As"));
        saveAsAction->setToolTip(tr("Save query criteria as file"));
        connect(saveAsAction, SIGNAL(triggered()), this, SLOT(onSaveAsActionTriggered()));

        QAction* openActtion = toolbar->addAction(APP->getResIcon("folder-open-o"), tr("Open"));
        openActtion->setToolTip(tr("Open query criteria from file"));
        connect(openActtion, SIGNAL(triggered()), this, SLOT(onOpenActtionTriggered()));

        QAction* saveAsCommonQueryAction = toolbar->addAction(APP->getResIcon("save-as"), tr("Save"));
        saveAsCommonQueryAction->setToolTip(tr("Save as a common query"));
        connect(saveAsCommonQueryAction, SIGNAL(triggered()), this, SLOT(onSaveAsCommonQueryActionTriggered()));

        // tab2-tab1-AdvancedQuery
        TAdvancedQuery *advancedQuery = new TAdvancedQuery(this);
        advancedQuery->setObjectName("AdvancedQuery");
        initAdvanceCfgMap();
        advancedQuery->setItemList(mAdvancedQueryCfgMap.value("advanced_items").toList());
        advancedQuery->setData(getDefaultQueryDataFromCfgMap());

        TScrollArea *scrollArea = new TScrollArea(this);
        scrollArea->setWidgetResizable(true);
        scrollArea->setWidget(advancedQuery);

        // tab2-tab1
        QMainWindow *mainWindow = new QMainWindow(this);
        mainWindow->addToolBar(toolbar);
        mainWindow->setCentralWidget(scrollArea);

        // tab2-tab2-TableView
        TTableView *commonQueryTable = new TTableView(this);
        commonQueryTable->setObjectName("CommonQueryTable");
        commonQueryTable->setShowGrid(false);
        commonQueryTable->setSelectionMode(QAbstractItemView::ExtendedSelection);
        commonQueryTable->verticalHeader()->setVisible(false);
        QVariantList headerItem;
        headerItem << QVariant() << TDataParse::variantList2Map(
                          QVariantList() <<"name" << "title" << "display" << tr("Query Name") << "resizeMode" << "Stretch" << "displayRole" << "$title");
        commonQueryTable->setHeaderItem(headerItem);
        commonQueryTable->setDataKeyList(QStringList() << "title" << "condition");
        commonQueryTable->setPrimaryKey("condition");
        connect(commonQueryTable, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(onCommonQueryTableDoubleClicked(const QModelIndex &)));

        QAction *deleleCommonQueryActtion = new QAction(APP->getResIcon("remove"), tr("Delete query"), this);
        connect(deleleCommonQueryActtion, SIGNAL(triggered()), this, SLOT(onDeleleCommonQueryActtionTriggered()));
        QMenu *contextMenu = new QMenu(this);
        contextMenu->addAction(deleleCommonQueryActtion);
        commonQueryTable->setContextMenu(contextMenu);

        // 刷新常用查询表格数据
        initCommonQueryMap();
        commonQueryTable->loadData(mAdvancedQueryTableDataMap.value("advance").toList());

        // 高级查询Tab
        TTabWidget *advancedQueryTabWidget = new TTabWidget(this);
        advancedQueryTabWidget->setObjectName("AdvancedQueryTabWidget");
        advancedQueryTabWidget->addTab(mainWindow, tr("Advanced Query"));
        advancedQueryTabWidget->addTab(commonQueryTable, tr("Common Query"));

        // 导航Tab
        mNaviView = new TTabWidget(this);
        TTabWidget *tabWidget = qobject_cast<TTabWidget *>(mNaviView);
        tabWidget->addTab(categoryTreeView, tr("Category"));
        tabWidget->addTab(advancedQueryTabWidget, tr("Advanced Query"));
        naviLayout->addWidget(mNaviView);
    } else if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        mNaviView = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(mNaviView);
        // uiLoader
        mNaviViewUi = new TUiLoader(this);
        mNaviViewUi->setSelf(this);
        mNaviViewUi->setScriptEngine(APP->scriptEngine());
        mNaviViewUi->setUiStr(ui("navi").toString());
        leftLayout->addWidget(mNaviViewUi);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftBtnsLayout->setMargin(0);
        leftBtnsLayout->setSpacing(10);
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), this, SLOT(onNaviReset()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refresh()));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        // categoryTreeView
        mNaviViewCat = new TCategoryTreeView(this);
        mNaviViewCat->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviViewCat->setVisible(false);
        }
        mNaviViewCat->setItemList(rootChildList);
        mNaviViewCat->setCheckable(config("navi.is_checkable").toBool());
        connect(mNaviViewCat, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        mNaviViewCat->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        leftLayout->addSpacing(6);
        leftLayout->addWidget(mNaviViewCat, 1);
        naviLayout->addWidget(mNaviView);
    } else {
        mNaviView = new TCategoryTreeView(this);
        mNaviView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        TCategoryTreeView *categoryTreeView = qobject_cast<TCategoryTreeView *>(mNaviView);
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        naviLayout->addWidget(mNaviView);
    }
}

void SysContactMgt::initTableView()
{
    mTableView = new TTableView(this);
    mTableView->setObjectName("TableView");
    mTableView->setStyleSheet("QTableView{border-width:0px}");
    mTableView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    mTableView->verticalHeader()->setVisible(false);
    mTableView->horizontalHeader()->setSortIndicatorShown(true);
    mTableView->horizontalHeader()->setSectionsMovable(true);
    mTableView->horizontalHeader()->setStretchLastSection(true);
    mTableView->setHeaderPopupEnabled(true);
    mTableView->horizontalHeader()->setMinimumSectionSize(75);

    mTableView->setDataKeyList(mTableConf.dataKeys);
    mTableView->setPrimaryKey(mTableConf.primaryKey);
    QVariantList headerItems = mTableConf.horizontalHeaders;
    headerItems.prepend(QVariant());
    mTableView->setHeaderItem(headerItems);
    connect(mTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)), this, SLOT(refresh()));
}

QVariantList SysContactMgt::parseCategoryConfig()
{
    QVariantList categoryLst;
    QVariantList categoryConfig = config("navi.categories").toList();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (format.compare("filter_by_enum") == 0) {
        for(const QVariant &item: categoryConfig) {
            QVariantMap itemMap = item.toMap();
            QString itemName         = itemMap.value("name").toString();
            QVariantList childrenLst = itemMap.value("children").toList();
            QString enumHook         = itemMap.value("enum_hook").toString();
            QString enumCfg          = itemMap.value("enum_children").toString();
            QStringList enumCheckedLst = itemMap.value("enum_checked").toStringList();
            QStringList enumInvisibleLst = itemMap.value("enum_invisible").toStringList();
            if (itemMap.contains("visible")) {
                itemMap.insert("VISIBLE", itemMap.take("visible"));
            }
            if (itemMap.contains("expand")) {
                itemMap.insert("EXPAND", itemMap.take("expand"));
            }
            TSqlWhereCompsiteV2 childWhere;
            childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
            QVariantList itemList = TOPENM->enumList(enumCfg)->toComboList();
            if (!enumHook.isEmpty() && isHookExists(enumHook)) {
                itemList = callHooksQuick(enumHook).toVariant().toList();
            }
            for (const QVariant &item: itemList) {
                QVariantMap enumMap = item.toMap();
                QString enumName = enumMap.value("name").toString();
                QVariantMap childMap;
                childMap.insert("name",enumName);
                childMap.insert("text",ttr(enumMap.value("text").toString()));
                childMap.insert("icon",enumMap.value("icon"));
                childMap.insert("data", itemName);
                if(enumCheckedLst.contains(enumName)) {
                    childMap.insert("checked",1);
                } else {
                    childMap.insert("checked",0);
                }
                if (enumInvisibleLst.contains(enumName)) {
                    childMap.insert("VISIBLE", 0);
                } else {
                    childMap.insert("VISIBLE",1);
                }
                childrenLst.append(childMap);
            }
            int checkedChildLen = 0;
            for (QVariant &item : childrenLst) {
                QVariantMap itemMap = item.toMap();
                QString name = itemMap.value("name").toString();
                if(itemMap.value("checked").toInt() == 1) {
                    childWhere.append(itemName, name);
                    checkedChildLen++;
                }
                item = itemMap;
            }
            if (checkedChildLen == childrenLst.length()) {
                itemMap.insert("checked", 1);
            } else {
                itemMap.insert("checked", 0);
            }
            mCategoryWhere.append(childWhere);

            itemMap.remove("children");
            itemMap.remove("enum_children");
            itemMap.remove("enum_checked");
            itemMap.remove("enum_invisible");
            itemMap.insert("text",ttr(itemMap.value("text").toString()));
            itemMap.insert("CHILDREN",childrenLst);

            categoryLst.append(itemMap);
        }
    } else if (format.compare("filter_by_sql_where") == 0) {
        return categoryConfig;
    }
    return categoryLst;
}

QVariantMap SysContactMgt::getCompanyMap()
{
    TSqlSelectorV2 selector;
    selector.setTable("pub_customer");
    selector.fieldRef() << "short_name" << "code" << "fullname";
    selector.addGroup("code,short_name,fullname");

    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    TDataResponse customerdataRes(data.toMap());
    if (customerdataRes.hasError()){
        TLOG_ERROR(customerdataRes.errText());
        return QVariantMap();
    }
    QVariantList customerData = customerdataRes.data().toList();

    QVariantMap companyMap;
    for (QVariant customerEle: customerData) {
        QVariantMap dataMap;
        dataMap.insert("short_name", customerEle.toMap().value("short_name").toString());
        dataMap.insert("fullname", customerEle.toMap().value("fullname").toString());
        companyMap.insert(customerEle.toMap().value("code").toString(),dataMap);
    }
    return companyMap;
}

void SysContactMgt::fillTableData(const TDataResponse &iDataRes)
{
    if (mPageTool != nullptr) {
        mPageTool->setRowCount(iDataRes.dataCount(), true);
    }
    QVariantList dataList = getTransferedTableData(iDataRes.data().toList());
    mTableView->loadData(dataList);
}

QVariantList SysContactMgt::getTransferedTableData(const QVariantList &iDataLst)
{
    QVariantList dataLst;
    //枚举转换
    QVariantMap companyMap = getCompanyMap();
    TEnumList *enumPeopleSex = TOPENM->enumList("sys-sex");
    TEnumList *enumContactStatus = TOPENM->enumList("sys-contact-status");
    TEnumList *enumContactDepartment = TOPENM->enumList("sys-contact-department");
    TEnumList *enumContactTitle = TOPENM->enumList("sys-contact-title");
    TEnumList *enumContactCategory = TOPENM->enumList("sys-contact-category");
    TEnumList *enumContactSource = TOPENM->enumList("sys-contact-source");

    for (QVariant row: iDataLst) {
        QVariantMap rowMap = row.toMap();
        if (enumPeopleSex != nullptr) {
            QString sexStr = rowMap.value("sex").toString();
            rowMap.insert("sex.icon", enumPeopleSex->itemIcon(sexStr));
            rowMap.insert("sex.text", enumPeopleSex->itemText(sexStr));
        }
        if (enumContactSource != nullptr) {
            QString sourceStr = rowMap.value("source").toString();
            rowMap.insert("source", enumContactSource->itemText(sourceStr));
        }
        if (enumContactStatus != nullptr) {
            QString statusStr = rowMap.value("status").toString();
            rowMap.insert("status", enumContactStatus->itemText(statusStr));
        }
        if (enumContactDepartment != nullptr) {
            QString departmentStr = rowMap.value("department").toString();
            rowMap.insert("department", enumContactDepartment->itemText(departmentStr));
        }
        if (enumContactTitle != nullptr) {
            QString titleStr = rowMap.value("title").toString();
            rowMap.insert("title", enumContactTitle->itemText(titleStr));
        }
        if (enumContactCategory != nullptr) {
            QString categoryStr = rowMap.value("category").toString();
            rowMap.insert("category", enumContactCategory->itemText(categoryStr));
        }

        //jsonb字段,根据PubComboLineCoupleWideget控件值格式
        QStringList tmpLst;
        QString address_work = rowMap["address_work"].toString();
        QString address_home = rowMap["address_home"].toString();
        QString address_other = rowMap["address_other"].toString();

        tmpLst = TDataParse::jsonStr2Variant(address_work).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["address_work"] = tmpLst.join(";");
        } else {
            rowMap["address_work"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(address_home).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["address_home"] = tmpLst.join(";");
        } else {
            rowMap["address_home"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(address_other).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["address_other"] = tmpLst.join(";");
        } else {
            rowMap["address_other"] = "";
        }

        QString phone_work = rowMap["phone_work"].toString();
        QString phone_mobile = rowMap["phone_mobile"].toString();
        QString phone_other = rowMap["phone_other"].toString();

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(phone_work).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["phone_work"] = tmpLst.join(";");
        } else {
            rowMap["phone_work"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(phone_mobile).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["phone_mobile"] = tmpLst.join(";");
        } else {
            rowMap["phone_mobile"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(phone_other).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["phone_other"] = tmpLst.join(";");
        } else {
            rowMap["phone_other"] = "";
        }

        QString fax_work = rowMap["fax_work"].toString();
        QString fax_other = rowMap["fax_other"].toString();

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(fax_work).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["fax_work"] = tmpLst.join(";");
        } else {
            rowMap["fax_work"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(fax_other).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["fax_other"] = tmpLst.join(";");
        } else {
            rowMap["fax_other"] = "";
        }

        QString mail = rowMap["mail"].toString();
        QString mail_personal = rowMap["mail_personal"].toString();

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(mail).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["mail"] = tmpLst.join(";");
        } else {
            rowMap["mail"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(mail_personal).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["mail_personal"] = tmpLst.join(";");
        } else {
            rowMap["mail_personal"] = "";
        }

        QString im_qq = rowMap["im_qq"].toString();
        QString im_msn = rowMap["im_msn"].toString();
        QString im_other = rowMap["im_other"].toString();//微信

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(im_qq).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["im_qq"] = tmpLst.join(";");
        } else {
            rowMap["im_qq"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(im_msn).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["im_msn"] = tmpLst.join(";");
        } else {
            rowMap["im_msn"] = "";
        }

        tmpLst.clear();
        tmpLst = TDataParse::jsonStr2Variant(im_other).toStringList();
        if (tmpLst.count() != 0) {
            rowMap["im_other"] = tmpLst.join(";");
        } else {
            rowMap["im_other"] = "";
        }

        rowMap.insert("company_short",companyMap.value(rowMap.value("company").toString()));
        dataLst.append(rowMap);
    }
    return dataLst;
}

QVariantList SysContactMgt::getAllContactList(QVariantList iExistIdLst)
{
    TSqlSelectorV2 selector;
    selector.setTable("pub_contacts");
    selector.fieldRef() << "id" << "name" << "status" << "sex" << "source"
                        <<  "source_serial" << "company" <<  "department"
                         << "title"  <<  "remark";

    QVariantList existLst = iExistIdLst;
    existLst << config("defaultData");
    selector.addWhere("id", existLst, "!=");
    selector.addOrder("id", Qt::DescendingOrder);
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));

    TDataResponse contactdataRes(data.toMap());
    if (contactdataRes.hasError()) {
        TLOG_ERROR(contactdataRes.errText());
        return QVariantList();
    }
    QVariantList contactData = contactdataRes.data().toList();
    return contactData;
}

void SysContactMgt::saveContactListInfo(QVariantList iDataLst)
{
    QVariantMap relationDataMap = getRelationData();
    QVariantList dbDataLst;
    for (QVariant ele: iDataLst) {
        QVariantMap rowMap;
        rowMap.insert(relationDataMap.value("relationTableIdName").toString(),config("defaultData"));
        rowMap.insert(relationDataMap.value("relationTableContactName").toString(),ele.toMap().value("id"));
        dbDataLst.append(rowMap);
    }
    QVariantMap argMap;
    argMap.insert("table", config("relationTable").toString());
    argMap.insert("field", QStringList() << relationDataMap.value("relationTableIdName").toString()
                  << relationDataMap.value("relationTableContactName").toString());
    argMap.insert("data", dbDataLst);

    QVariant data = doThreadWork(new SysontactMgtThread(this),"BATCH_INSERT" , argMap);
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        TLOG_ERROR(dataRes.errText());
        TMessageBox::error(this,ttr("Save data failed!"));
        return;
    } else {
        TMessageBox::info(this,ttr("Save data success!"));
        refresh();
    }
}

QStringList SysContactMgt::listRelationCustomerId()
{
    QStringList customerIds;
    QVariantMap relationDataMap = getRelationData();
    if (!relationDataMap.isEmpty()) {
        TSqlSelectorV2 selector;
        selector.setTable(config("relationTable").toString());
        selector.setField(relationDataMap.value("relationTableContactName").toString());
        selector.setWhere(relationDataMap.value("relationTableIdName").toString(),config("defaultData").toString());

        QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYVALUE, QVariant::fromValue(selector));
        TDataResponse dataRes(data.toMap());
        if (dataRes.hasError()) {
            TLOG_ERROR(dataRes.errText());
            return QStringList();
        } else {
            for (QVariant row: dataRes.data().toList()) {
                customerIds.append(row.toString());
            }
        }
    }

    return customerIds;
}

QVariantMap SysContactMgt::getRelationData()
{
    if (!APP->isBufferDataExists(RELATEION_DATA_KEY)) {
        QVariantMap relationData;
        if (!config("relationTable").toString().isEmpty() && !config("defaultData").toString().isEmpty()) {
            QVariant data = doThreadWork(new SysontactMgtThread(this), "GET_TABLE_FIELD", config("relationTable").toString());
            TDataResponse dataRes(data.toMap());
            if (dataRes.hasError()) {
                TLOG_ERROR(dataRes.errText());
                return QVariantMap();
            }

            QStringList lst = dataRes.data().toStringList();
            if (!lst.isEmpty() && (lst.count() > 1)) {
                relationData.insert("relationTableIdName", lst.first());
                relationData.insert("relationTableContactName", lst.at(1));
            }
            APP->setBufferData(RELATEION_DATA_KEY,relationData);
        }
    }
    return APP->bufferData(RELATEION_DATA_KEY).toMap();
}

/**
* \brief 初始化高级查询配置数据
* mAdvancedQueryCfgMap结构如下：
* \code
    advance: {
      "advanced_items" : [
        {"name" : "attr_data.machine_code", "title" : "Machine Code" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "attr_data.machine_name", "title" : "Machine Name" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "title", "title" : "Title" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "class", "title" : "Class" , "widgetType" : "ComboBox" , "wgt_prop" : "" , "optionList" : "enum(tpm-machine-maintenance-plan-class)"}
      ],
      "file_suffix": ".conf"
      "condition" :{
        "condition": "1 and 2",
        "value": [
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_code",
                "operator": "LIKE",
                "title": "Machine Code",
                "valid": true,
                "value1": "00"
            },
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_name",
                "operator": "LIKE",
                "title": "Machine Name",
                "valid": true,
                "value1": "4"
            }
        ]
      }
    }
* \endcode
*/
void SysContactMgt::initAdvanceCfgMap()
{
    mAdvancedQueryCfgMap = config("advance").toMap();
    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList newQueryItems;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        // 翻译title
        m.insert("title", ttr(m.value("title").toString()));
        // 转换枚举
        QVariantList optionList;
        QRegExp regExp("enum(.*)");
        if ((m.value("optionList").type() == QVariant::String) && (regExp.indexIn(m.value("optionList").toString()) != -1)) {
            if (regExp.capturedTexts().count() > 1) {
                optionList = TOPENM->enumList(regExp.capturedTexts().value(1).remove("(").remove(")"))->toComboList();
                m.insert("optionList", optionList);
            }
        }
        newQueryItems << m;
    }
    mAdvancedQueryCfgMap.insert("advanced_items", newQueryItems);
}

/**
* \brief 获取高级查询控件默认的查询条件
*/
QVariantMap SysContactMgt::getDefaultQueryDataFromCfgMap()
{
    QVariantMap conditionMap = config("advance").toMap().value("condition").toMap();
    QVariantList valueList = conditionMap.value("value").toList();
    for (QVariant &var: valueList) {
        QVariantMap m = var.toMap();
        m.insert("title", ttr(m.value("title").toString()));
        var = m;
    }
    conditionMap.insert("value", valueList);
    return conditionMap;
}

/**
* \brief 从高级查询配置中获取未使用条目的name和text，添加条目时使用
*/
QVariantList SysContactMgt::getAdvancedQueryFieldList()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return QVariantList();
    }

    QVariantList selectedFieldDataLst = advancedQuery->getQueryData().value("value").toList();
    QStringList selectedFieldLst;
    for (QVariant val : selectedFieldDataLst){
        selectedFieldLst.append(val.toMap().value("name").toString());
    }

    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList fieldList;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        QVariantMap itemMap;
        if(!selectedFieldLst.contains(m.value("name").toString())) {
            itemMap.insert("name", m.value("name").toString());
            itemMap.insert("title", ttr(m.value("title").toString()));
            fieldList << QVariant(itemMap);
        }
    }
    return fieldList;
}

/**
* \brief 从sys_conf表查找出已保存的常用查询数据
* 数据结构如下：
* \code
{
  "advance": [
    {
      "condition": {
        "condition": "1 and 2",
        "value": [
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_code",
            "operator": "LIKE",
            "title": "设备编号",
            "valid": true,
            "value1": "00"
          },
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_name",
            "operator": "LIKE",
            "title": "设备名称",
            "valid": true,
            "value1": "4"
          }
        ]
      },
      "title": "1"
    }
  ]
}
* \endcode
*/
void SysContactMgt::initCommonQueryMap()
{
    TSqlSelectorV2 selector;
    selector.setTable("sys_conf");
    selector.setField(QStringList() << "data");
    selector.setWhere("path", QStringList() << this->moduleName() << "AdvancedQuery", "@>");
    TError err;
    QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                            QVariant::fromValue(selector),
                                            &err);
    if (err.isValid()) {
        alertError(ttr("Load data failed!"), err.text());
        return;
    }
    mAdvancedQueryTableDataMap = TDataParse::jsonStr2Variant(data.toMap().value("data").toString()).toMap();
}

/**
* \brief 根据objname从导航栏中找到高级查询控件
*/
TAdvancedQuery *SysContactMgt::findAdvancedQuery()
{
    return mNaviView->findChild<TAdvancedQuery *>("AdvancedQuery");
}

/**
* \brief 根据objname从导航栏中找到常用查询tableView控件
*/
TTableView *SysContactMgt::findCommonQueryTable()
{
    return mNaviView->findChild<TTableView *>("CommonQueryTable");
}

/**
* \brief 根据objname从导航栏中找到高级查询TabWidget控件
*/
TTabWidget *SysContactMgt::findAdvancedQueryTabWidget()
{
    return mNaviView->findChild<TTabWidget *>("AdvancedQueryTabWidget");
}

SysContactMgt::NaviType SysContactMgt::str2NaviType(const QString &iType)
{
    const QString type = iType.toUpper();
    if (type == "UILOADER") {
        return NaviType::UiLoader;
    } else if (type == "CATEGORYTREEVIEWANDADVANCEDQUERY") {
        return NaviType::CategoryTreeViewAndAdvancedQuery;
    } else if (type == "UILOADERANDCATEGORYTREEVIEW") {
        return NaviType::UiLoaderAndCategoryTreeView;
    } else if (type == "NONE") {
        return NaviType::None;
    } else {
        return NaviType::CategoryTreeView;
    }
}

QString SysContactMgt::uiloaderWhere() const
{
    TUiLoader *filterUi = nullptr;
    if (mNaviType == NaviType::UiLoaderAndCategoryTreeView) {
        filterUi = mNaviViewUi;
    } else {
        filterUi = qobject_cast<TUiLoader*>(mNaviView);
    }
    if (filterUi == nullptr) {
        return QString();
    }

    auto filterDataMap = filterUi->getAllValues().toVariant().toMap();

    QString searchSql;
    QMapIterator<QString, QVariant> iter(filterDataMap);
    while (iter.hasNext()) {
        iter.next();
        QString key = iter.key();
        QString value = iter.value().toString();
        if (value.isEmpty()) {
            continue;
        }
        auto obj = filterUi->getObject(key);
        if (obj != nullptr) {
            QVariantMap userData = obj->property("tui_user_data").toMap();
            if (userData.contains("field_name")) {
                key = userData.value("field_name").toString();
            }
            QString operatorStr = "=";
            if (userData.contains("operator")) {
                operatorStr = userData.value("operator").toString();
            }
            if (operatorStr.compare("like", Qt::CaseInsensitive) == 0 ||
                    operatorStr.compare("ilike", Qt::CaseInsensitive) == 0) {
                value = QString("'%%1%'").arg(value);
            } else if (operatorStr.compare("in", Qt::CaseInsensitive) == 0) {
                value = QString("(%1)").arg(value);
            } else {
                if (iter.value().type() == QVariant::String) {
                    value = QString("'%1'").arg(value);
                }
            }
            QString searchStr = QString("%1 %2 %3").arg(key).arg(operatorStr).arg(value);
            if (!searchSql.isEmpty()) {
                searchSql.append(" and ");
            }
            searchSql.append(searchStr);
        }
    }
    return searchSql;
}
