﻿#ifndef PUBCONTACTMGT_H
#define PUBCONTACTMGT_H

#include <tbaseutil/tdataresponse.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>
#include "syscontact.h"

class TAdvancedQuery;
class TCategoryTreeView;
class TPageTool;
class TSearchEntry;
class TSplitter;
class TTableView;
class TTabWidget;

class SysContactMgt : public TopClassAbs
{
    Q_OBJECT
public:
    explicit SysContactMgt(const QString &iModuleNameStr = QString(""),
                           const QVariantMap iUrlPars = QVariantMap(),
                           QWidget *iParent = nullptr);
    ~SysContactMgt();
    enum class NaviType {
        None,
        CategoryTreeView,
        UiLoader,
        CategoryTreeViewAndAdvancedQuery,
        UiLoaderAndCategoryTreeView
    };
public slots:
    void refresh(bool iResetPageBol = true);
    void deleteItem(const QVariantList &iIdLst);
    QVariantList selectedItems();
    TopClassAbs *openItem(int iUserId);
    void newItem();
    void addContacts();
    bool removeItem(int iItemId);
    bool canModify();

private slots:
    void onPageChanged();
    void onSelectionChanged();
    void onDetailSaved(const QVariant &iUidVar);
    void onCategoryViewDataChanged(QVariantList iDataLst);
    void onDetailChanged();
    void onAddActionTriggered();
    void onClearActionTriggered();
    void onSaveAsActionTriggered();
    void onOpenActtionTriggered();
    void onSaveAsCommonQueryActionTriggered();
    void onCommonQueryTableDoubleClicked(const QModelIndex &index);
    void onDeleleCommonQueryActtionTriggered();
    void onNaviReset();

private:
    TSqlSelectorV2 getSqlSelector(bool iResetPage = true);
    void initNaviView();
    void initTableView();
    QVariantList parseCategoryConfig();
    QVariantMap getCompanyMap();
    void fillTableData(const TDataResponse &iDataRes);
    QVariantList getTransferedTableData(const QVariantList &iDataLst);
    QVariantList getAllContactList(QVariantList iExistIdLst);
    void saveContactListInfo(QVariantList iDataLst);
    QStringList listRelationCustomerId();
    QVariantMap getRelationData();

    void initAdvanceCfgMap();
    QVariantMap getDefaultQueryDataFromCfgMap();
    QVariantList getAdvancedQueryFieldList();
    void initCommonQueryMap();
    TAdvancedQuery *findAdvancedQuery();
    TTableView *findCommonQueryTable();
    TTabWidget *findAdvancedQueryTabWidget();
    NaviType str2NaviType(const QString &iType);
    QString uiloaderWhere() const;

private:
    TSplitter *mBodySplitter = nullptr;
    TSplitter *mBodyWidget = nullptr;
    TTableView *mTableView = nullptr;
    TSearchEntry *mSearchEntry = nullptr;
    TPageTool *mPageTool = nullptr;
    QVariantList mSelectedLst = QVariantList();
    SysContact *mDetailView = nullptr;
    TCategoryTreeView *mNaviViewCat = nullptr;
    TUiLoader *mNaviViewUi = nullptr;
    NaviType mNaviType = NaviType::CategoryTreeView;
    QWidget *mNaviView = nullptr;
    TSqlWhereCompsiteV2 mCategoryWhere;
    QVariantMap mAdvancedQueryCfgMap;
    QVariantMap mAdvancedQueryTableDataMap;
    TopClassTableConf mTableConf;
    QString mFixedSqlWhere = "";
};

#endif // PUBCONTACTMGT_H
