﻿#ifndef SYSLOGCONSOLE_H
#define SYSLOGCONSOLE_H

#include <QModelIndex>
#include <topcore/topclassabs.h>

class QCheckBox;
class QToolBar;
class TAction;
class TMenu;
class TSearchEntry;
class TTableModel;
class TTableView;
class TVBoxLayout;

class SysLogConsole : public TopClassAbs
{
    Q_OBJECT
public:
    explicit SysLogConsole(const QString &iModuleNameStr = QString(""),
                           const QVariantMap iUrlPars = QVariantMap(),
                           QWidget *iParent = nullptr);
    ~SysLogConsole();

public slots:
    void appendLog(const QString &iTextStr, const QString &iTypeStr, const QString &iDateTimeStr,const QString &iSrcPosStr, const QString &iDebugLevelStr);
    void clearTable();
    void filter();
    void onDoSearch(QString iKeyStr, QVariant iFilterVar);
    void setDebugLevel(int iDebugLevel);
    int debugLevel() const;
    void refresh();

private slots:
    void setErrorState(bool iBol);
    void setWarningState(bool iBol);
    void setInfoState(bool iBol);
    void setSqlState(bool iBol);
    void setJsState(bool iBol);
    void setOtherState(bool iBol);
    void setHttpDebug(bool iBol);
    void logTableDoubleClick(QModelIndex iModelIndex);

private:
    void initTableView();
    void initHistoryLog();

private:
    TTableView *mLogTableView;
    TTableModel *mLogTableModel;
    QStringList mTableDataKeyLst;

    TSearchEntry *mSearchEntry;
    QToolBar *mToolbar;
    TAction *mFilter;

    TMenu *mFilterMenu;
    QAction *mErrorAction;
    TVBoxLayout *mFilterLayout;
    QCheckBox *mErrorCheck;
    QCheckBox *mWarningCheck;
    QCheckBox *mInfoCheck;
    QCheckBox *mSqlCheck;
    QCheckBox *mJSCheck;
    QCheckBox *mOtherCheck;
    QCheckBox *mWebDebugCheck;
    int mOldDebugMode;
};

#endif // SYSLOGCONSOLE_H
