﻿#include "topdesktopabs.h"
#include "topdesktopgroupabs.h"
#include "topdesktopitemabs.h"
#include "topuim.h"
#include "topcore.h"
#include "topdataaccessmanager.h"
#include <QWidget>
#include <QWheelEvent>
#include <QScrollBar>
#include <QMimeData>
#include <QToolBar>
#include <QFileDialog>
#include <QGuiApplication>
#include <QImageReader>
#include <QPainter>
#include <QToolTip>
#include <TBaseutil/tthemestyle.h>
#include <TBaseutil/tdataparse.h>
#include <TBaseutil/tresource.h>
#include <TBaseutil/tfileio.h>
#include <TWidget/tscrollarea.h>
#include <TWidget/tsearchentry.h>
#include <TWidget/tuiloaderdialog.h>
#include <TWidget/tuiloader.h>
#include <TWidget/tmessagebox.h>
#include <TWidget/tcombobox.h>
#include <TWidget/ttableviewdialog.h>
#include <TWidget/tmessagebar.h>
#include <TWidget/tlineedit.h>



enum WndSize
{
    WndSize_CellWidth   = 95,
    WndSize_CellHeight  = 95,
    WndSize_WndWidth    = 800,
    WndSize_WndHeight   = 600,
    WndSize_DelayTime   = 200,
    WndSize_Spacing     = 20,
    WndSize_Margin      = 5
};

class TopDesktopAbsPrivate
{
    Q_DECLARE_PUBLIC(TopDesktopAbs)
public:
    TopDesktopAbsPrivate(TopDesktopAbs *qptr);

    bool mIsEditingState;
    QVariantMap mDataVarMap;
    QList<TopDesktopGroupAbs *> mGroupsLst;
    bool mIsDataModifiedBol;
    TMessageBar *mMessageBar;
    bool mIsFromDbBol;
protected:
    TopDesktopAbs * const q_ptr;
};


/*****************************************/

TopDesktopAbsPrivate::TopDesktopAbsPrivate(TopDesktopAbs *qptr) :
    mIsEditingState(false),
    mIsDataModifiedBol(false),
    mIsFromDbBol(false),
    q_ptr(qptr)
{

}



TopDesktopAbs::TopDesktopAbs(QWidget *iParent) :
    QFrame(iParent),
    d_ptr(new TopDesktopAbsPrivate(this))
{
    Q_D(TopDesktopAbs);
//    setProperty("SS", "DESKTOP");
    d->mMessageBar = new TMessageBar(this);
    d->mMessageBar->hide();
    d->mMessageBar->raise();
}

TopDesktopAbs::~TopDesktopAbs()
{
    Q_D(TopDesktopAbs);

    qDeleteAll(d->mGroupsLst);
}

void TopDesktopAbs::doLayout()
{

}

void TopDesktopAbs::refresh()
{

}

void TopDesktopAbs::fromDb(bool iFromDbBol)
{
    Q_D(TopDesktopAbs);
    d->mIsFromDbBol = iFromDbBol;
}

bool TopDesktopAbs::isFromDb()
{
    Q_D(TopDesktopAbs);
    return d->mIsFromDbBol;
}

QList<TopDesktopGroupAbs *> TopDesktopAbs::desktopGroups()
{
    Q_D(TopDesktopAbs);

    return d->mGroupsLst;
}

void TopDesktopAbs::setEditable(bool iEditableBol)
{
    Q_D(TopDesktopAbs);

    d->mIsEditingState = iEditableBol;
    qDebug() << d->mGroupsLst.count() << iEditableBol;
    foreach (TopDesktopGroupAbs *group, d->mGroupsLst) {
        group->setEditable(iEditableBol);
    }
}

bool TopDesktopAbs::editable()
{
    Q_D(TopDesktopAbs);
    return d->mIsEditingState;
}

void TopDesktopAbs::setConfigData(const QVariantMap &iConfigDataVarMap)
{
    Q_D(TopDesktopAbs);
    d->mDataVarMap = iConfigDataVarMap;

    if (iConfigDataVarMap.contains("groups"))
    {
        foreach (TopDesktopGroupAbs *group, d->mGroupsLst) {
            delete group;
        }
        d->mGroupsLst.clear();
        d->mGroupsLst = createGroups(iConfigDataVarMap.value("groups").toList());
    }

    if (!isFromDb() && editable()) update();

    setBackground();
}

QVariantMap TopDesktopAbs::configData()
{
    Q_D(TopDesktopAbs);
    QVariantMap dataVarMap = d->mDataVarMap;
    QVariantList groupsVarLst;
    foreach (TopDesktopGroupAbs *group, d->mGroupsLst) {
        groupsVarLst.push_back(group->configData());
    }
    dataVarMap.insert("groups", groupsVarLst);
    if (!dataVarMap.contains("background")) dataVarMap.insert("background", "");
    if (!dataVarMap.contains("backgroundImage")) dataVarMap.insert("backgroundImage", "");

    return dataVarMap;
}

void TopDesktopAbs::setConfig(const QString &iKeyStr, const QVariant &iValueVar)
{
    Q_D(TopDesktopAbs);

    d->mDataVarMap.insert(iKeyStr, iValueVar);
    if (iKeyStr == "size" && !isFromDb() && editable()) update();
}

QVariant TopDesktopAbs::config(const QString &iKeyStr) const
{
    Q_D(const TopDesktopAbs);
    return d->mDataVarMap.value(iKeyStr);
}

bool TopDesktopAbs::removeOne(TopDesktopGroupAbs *iDesktopGroup)
{
    Q_D(TopDesktopAbs);
    return d->mGroupsLst.removeOne(iDesktopGroup);
}

void TopDesktopAbs::appendOne(TopDesktopGroupAbs *iDesktopGroup)
{
    Q_D(TopDesktopAbs);
    if (iDesktopGroup) d->mGroupsLst.append(iDesktopGroup);
}

void TopDesktopAbs::onDoSearch(const QString &iTextStr)
{
    if (!iTextStr.startsWith('<'))
    {
        doLayout();
    }
    else
    {
        if (iTextStr.endsWith('>')
                && APP->userName().toUpper() == "ADMIN")
        {
            emit urlRequested(iTextStr.mid(1, iTextStr.length() - 2));
        }
    }
}

void TopDesktopAbs::onDataChanged()
{
    setDataModified(true);
}

void TopDesktopAbs::setDataModified(bool iModifedBol)
{
    Q_D(TopDesktopAbs);
    d->mIsDataModifiedBol = iModifedBol;
    if (iModifedBol) emit dataChanged();
}

bool TopDesktopAbs::isDataModified() const
{
    Q_D(const TopDesktopAbs);
    return d->mIsDataModifiedBol;
}

QSize TopDesktopAbs::sizeStr2QSize(const QString &iSizeStr)
{
    QString rowStr;
    QString colStr;
    QString sizeStr = iSizeStr;
    if (sizeStr.contains('x'))
    {
        QStringList sizeStrLst = sizeStr.split('x');
        rowStr = sizeStrLst.at(0);
        colStr = sizeStrLst.at(1);
    }
    if (rowStr.isEmpty()) rowStr = "0";
    if (colStr.isEmpty()) colStr = "0";

    QSize size(rowStr.toInt(), colStr.toInt());
    return size;
}

QString TopDesktopAbs::rowColStr2SizeStr(const QString &iRowStr, const QString &iColStr)
{
    return QString("%1x%2").arg(iRowStr).arg(iColStr);
}

TMessageBar *TopDesktopAbs::messageBar()
{
    Q_D(TopDesktopAbs);
    return d->mMessageBar;
}

void TopDesktopAbs::setBackground()
{

}

QList<TopDesktopGroupAbs *> TopDesktopAbs::createGroups(const QVariantList &iGroupsVarLst)
{
    QList<TopDesktopGroupAbs *> groupsLst;
    foreach (QVariant groupVar, iGroupsVarLst) {
        groupsLst.push_back(createGroup(groupVar));
    }

    return groupsLst;
}

TopDesktopGroupAbs *TopDesktopAbs::createGroup(const QVariant &/*iGroupVar*/)
{
    return 0;
}

void TopDesktopAbs::resizeEvent(QResizeEvent *iEvent)
{
    doLayout();
    QFrame::resizeEvent(iEvent);
}


/*****************************************/
/*****************************************/

enum DesktopAction
{
    DesktopAction_Refresh       = 0,
    DesktopAction_SetEditable   = 1,
    DesktopAction_SaveModified  = 2,
    DesktopAction_SetBackground = 3,
    DesktopAction_AddGroup      = 4
};

class TopWin8DesktopPrivate
{
    Q_DECLARE_PUBLIC(TopWin8Desktop)
public:
    TopWin8DesktopPrivate(TopWin8Desktop *qptr);

    TScrollArea *mScrollArea;
    QWidget *mScrollCenterWgt;
    TSearchEntry *mSearchEntry;
    QToolBar *mToolBar;
    TopDesktopGroupAbs *mRightClickedGroup;
    QMenu *mGroupMenu;
    QMenu *mDesktopMenuFromDb;
    QMenu *mDesktopMenu;
    QMap<QString, QString> mLanguageMap;
    QMap<DesktopAction, QAction *> mDesktopActionsFromDb;

protected:
    TopWin8Desktop * const q_ptr;
};


TopWin8DesktopPrivate::TopWin8DesktopPrivate(TopWin8Desktop *qptr)
    : mRightClickedGroup(NULL),
      mGroupMenu(NULL),
      mDesktopMenuFromDb(NULL),
      q_ptr(qptr)
{

}

/*****************************************/

TopWin8Desktop::TopWin8Desktop(QWidget *iParent) :
    TopDesktopAbs(iParent),
    d_ptr(new TopWin8DesktopPrivate(this))
{
    Q_D(TopWin8Desktop);

    this->setMouseTracking(true);
    this->resize(TTHEME_SCALE(WndSize_WndWidth), TTHEME_SCALE(WndSize_WndHeight));
    this->setObjectName("DESKTOP");
    this->setStyleSheet("background-color:red;");
    d->mScrollArea = new TScrollArea(this);
    d->mScrollArea->setObjectName("SCROLLAREA");
    d->mScrollCenterWgt = new QWidget(d->mScrollArea);
    d->mScrollCenterWgt->setObjectName("CENTERWIDGET");
    d->mScrollArea->setWidget(d->mScrollCenterWgt);
    d->mScrollArea->setWidgetResizable(true);

    QVBoxLayout *centerLayout = new QVBoxLayout(this);
    centerLayout->setMargin(0);
    centerLayout->setSpacing(0);
    setLayout(centerLayout);

    TopUim *uim = new TopUim(this);

    uim->setUimStr("<ui> \n"
                   "<toolbar name='MainToolBar'> \n"
                   "<stretcher/> \n"
                   "<searchentry name=\"SEARCH_ENTRY\"/> \n"
                   "</toolbar> \n"
                   "</ui>");
    QToolBar *toolbar = qobject_cast<QToolBar*>(uim->getWidget("MainToolBar"));
    if (toolbar)
    {
        toolbar->setProperty("SS", "TITLE");
        toolbar->setStyleSheet(".Q{}");
        toolbar->installEventFilter(this);
        toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        toolbar->setMovable(false);
    }
    centerLayout->addWidget(toolbar, 0, Qt::AlignTop);
    centerLayout->addWidget(d->mScrollArea, 1);
    TSearchEntry * mSearchEntry = qobject_cast<TSearchEntry *>(uim->getWidget("MainToolBar/SEARCH_ENTRY"));
    if (mSearchEntry)
    {
        mSearchEntry->setPlaceholderText(tr("Search"));
        mSearchEntry->setSearchDelayTime(WndSize_DelayTime);
        connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(onDoSearch(QString)));
        d->mSearchEntry = mSearchEntry;
    }

// 分组Menu
    QMenu *mGroupMenu = new QMenu(this);
    mGroupMenu->addAction(TRES->icon("edit2"), tr("Edit"), this, SLOT(editGroup()));
    mGroupMenu->addAction(TRES->icon("delete"), tr("Delete"), this, SLOT(deleteGroup()));
    mGroupMenu->addAction(TRES->icon("add"), tr("Add Module"), this, SLOT(addItem()));
    d->mGroupMenu = mGroupMenu;
// 桌面Menu
    QMenu *mDesktopMenuFromDb = new QMenu(this);
    // 刷新
    QAction *refreshAction = new QAction(TRES->icon("refresh"), tr("Refresh"), this);
    connect(refreshAction, SIGNAL(triggered()), this, SLOT(refresh()));
    d->mDesktopActionsFromDb.insert(DesktopAction_Refresh, refreshAction);
    mDesktopMenuFromDb->addAction(refreshAction);
    // 编辑桌面
    QAction *editAction = new QAction(TRES->icon("edit2"), tr("Edit Desktop"), this);
    connect(editAction, SIGNAL(triggered()), this, SLOT(editDesktop()));
    d->mDesktopActionsFromDb.insert(DesktopAction_SetEditable, editAction);
    mDesktopMenuFromDb->addAction(editAction);
    // 保存
    QAction *saveAction = new QAction(TRES->icon("save"), tr("Save Desktop"), this);
    connect(saveAction, SIGNAL(triggered()), this, SLOT(saveDesktop()));
    d->mDesktopActionsFromDb.insert(DesktopAction_SaveModified, saveAction);
    mDesktopMenuFromDb->addAction(saveAction);
    // 设置背景颜色
    QAction *setBgAction = new QAction(TRES->icon("setting2"), tr("Set Background"), this);
    connect(setBgAction, SIGNAL(triggered()), this, SLOT(setDesktopBackground()));
    d->mDesktopActionsFromDb.insert(DesktopAction_SetBackground, setBgAction);
    mDesktopMenuFromDb->addAction(setBgAction);
    // 添加分组
    QAction *addGroupAction = new QAction(TRES->icon("add"), tr("Add Group"), this);
    connect(addGroupAction, SIGNAL(triggered()), this, SLOT(addGroup()));
    d->mDesktopActionsFromDb.insert(DesktopAction_AddGroup, addGroupAction);
    mDesktopMenuFromDb->addAction(addGroupAction);

    d->mDesktopMenuFromDb = mDesktopMenuFromDb;


    // 来自设置的数据
    QMenu *mDesktopMenuFromConfig = new QMenu(this);
    mDesktopMenuFromConfig->addAction(TRES->icon("setting2"), tr("Edit Config"), this, SLOT(editDesktopPopupConfig()));
    mDesktopMenuFromConfig->addAction(TRES->icon("add"), tr("Add Group"), this, SLOT(addGroup()));
    d->mDesktopMenu = mDesktopMenuFromConfig;

    d->mScrollArea->setStyleSheet(
                    QString(".TScrollArea#SCROLLAREA{border-image:url()}\n" \
                            ".QWidget#CENTERWIDGET{background:transparent}\n" \
                            "QScrollBar::corner{border:0px none}\n" \
                            "QScrollBar:horizontal, QScrollBar:vertical{background:rgba(255, 255, 255, 0.3);margin:2px; height:14px;border-radius:5px;border:0px none} \n" \
                            "QScrollBar::handle:horizontal, QScrollBar::handle:vertical{background:rgba{255, 255, 255, 0.3};margin:0px;border:0px none; border-radius:5px} \n" \
                            "QScrollBar::add-line:horizontal, QScrollBar::add-line:vertical{background:transparent;width:0px;} \n" \
                            "QScrollBar::sub-line:horizontal, QScrollBar::sub-line:vertical{background:transparent;width:0px;} \n" \
                            "QScrollBar::add-page:vertical, QScrollBar::add-page:vertical{background:transparent;width:0px}"));


    loadLanguagePkg();
    setBackground();

    this->setAcceptDrops(true);
    this->installEventFilter(this);
    d->mScrollArea->setMouseTracking(true);
    d->mScrollCenterWgt->setMouseTracking(true);
}

TopWin8Desktop::~TopWin8Desktop()
{
}

void TopWin8Desktop::doLayout()
{
    Q_D(TopWin8Desktop);

    int deskcateHeightInt = 0;
    int deskcateWidthInt = 0;
    QString searchTextStr = d->mSearchEntry->text();
    if (searchTextStr.endsWith('>')) searchTextStr = "";
    QRegExp reg(QRegExp::escape(searchTextStr), Qt::CaseInsensitive);
    foreach (TopDesktopGroupAbs *category, desktopGroups()) {
        foreach (TopDesktopItemAbs *item, category->desktopItems()) {
            if (item->hasOpenRight()
                        && (searchTextStr.isEmpty()
                            || reg.indexIn(item->config("title").toString()) >= 0
                            || reg.indexIn(item->config("title_zhcn").toString()) >= 0
                            || reg.indexIn(item->config("title_zhtw").toString()) >= 0
                            || reg.indexIn(item->config("title_en").toString()) >= 0
                            || reg.indexIn(item->config("title_jp").toString()) >= 0
                            || reg.indexIn(item->config("title_ot").toString()) >= 0))
            {
                item->setProperty("VISIBLE", QVariant(true));
            }
            else
            {
                item->setProperty("VISIBLE", QVariant(false));
            }
        }

        category->blockSignals(true);
        category->doLayout();
        category->show();
        category->blockSignals(false);

        category->move(category->point());
        if (category->geometry().bottom() > deskcateHeightInt) deskcateHeightInt = category->geometry().bottom();
        if (category->geometry().right() > deskcateWidthInt) deskcateWidthInt = category->geometry().right();
    }

    // move 分组位置, 加上Margin值是为了 让分组不会贴近边缘，以免看起来感觉并不好
    d->mScrollCenterWgt->setFixedSize(deskcateWidthInt + TTHEME_SCALE(WndSize_Margin),
                                      deskcateHeightInt + TTHEME_SCALE(WndSize_Spacing));
}

QString TopWin8Desktop::ttr(const QString &iStr)
{
    Q_D(TopWin8Desktop);
    return d->mLanguageMap.value(iStr, iStr);
}

void TopWin8Desktop::setColorList(TComboBox *iBackgroundCombox)
{
    QString colorsStr =  "["
                         "\"rgba(42, 105, 238, 0.9)\","
                         "\"rgba(239, 59, 5, 0.9)\","
                         "\"rgba(255, 153, 0, 0.9)\","
                         "\"rgba(27, 117, 27, 0.9)\","
                         "\"rgba(110, 9, 189, 0.9)\","
                         "\"rgba(0, 204, 204, 0.9)\","
                         "\"rgba(51, 153, 51, 0.9)\","
                         "\"rgba(250, 123, 56, 0.9)\","
                         "\"rgba(86, 97, 153, 0.9)\","
                         "\"rgba(173, 86, 94, 0.9)\","
                         "\"rgba(128,128,255,0.9)\","
                         "\"rgba(164,99,255,0.9)\","
                         "\"rgba(0,128,128,0.9)\","
                         "\"rgba(204,51,51,0.9)\","
                         "\"rgba(255, 191, 24, 0.9)\""
                         "]";
    QVariantList btnBkgColorVarLst = TDataParse::jsonStr2Variant(colorsStr).toList();
    //1. 下拉框添加颜色字符
    QVariantList colorVarLst;
    foreach(QVariant itemVar, btnBkgColorVarLst)
    {
        QString colorStr = itemVar.toString();

        QVariantMap comboVarMap;
        comboVarMap.insert("name", colorStr);
        comboVarMap.insert("text", colorStr);

        colorVarLst << comboVarMap;
    }
    iBackgroundCombox->setItemList(colorVarLst);

    //2. 设置颜色图标
    for(int n = 0; n < colorVarLst.count(); n++)
    {
        QVariantMap itemVarMap = colorVarLst.at(n).toMap();
        QString colorStr = itemVarMap.value("text").toString();

        int rInt = 0, gInt = 0, bInt = 0;
        QRegExp rx("(\\d+)\\s*\\,\\s*(\\d+)\\s*\\,\\s*(\\d+)");
        if (rx.indexIn(colorStr) != -1)
        {
            rInt = rx.cap(1).toInt();
            gInt = rx.cap(2).toInt();
            bInt = rx.cap(3).toInt();
        }

        //创建pixmap
        const int widthInt = 80;
        QPixmap pixColor(widthInt, 20);
        pixColor.fill(QColor(rInt, gInt, bInt, 210));

        iBackgroundCombox->setItemIcon(n, QIcon(pixColor));
        iBackgroundCombox->setIconSize(QSize(widthInt, 20));
    }
}

QString TopWin8Desktop::showUrlDialog()
{
    TTableViewDialog *dialog = new TTableViewDialog(this);
    dialog->resize(600, 400);
    dialog->setTitle(tr("Select URL Address"));
    dialog->setButtons(QStringList() << "Ok" << "Cancel"
                       <<tr("Unselect All")+":UnselectAll:ResetRole");

    QVariantList headeritemsVarLst;
    headeritemsVarLst << QVariant();
    headeritemsVarLst << TDataParse::stringList2VariantMap(
                       QStringList() << "name" << "name" << "display" << ttr("Class Name") <<
                       "resizeMode" << "Stretch" << "size"<< "80" <<
                       "displayRole" << "$name");

    headeritemsVarLst << TDataParse::stringList2VariantMap(
                       QStringList() << "name" << "disp" << "display" << ttr("Module Name") <<
                       "resizeMode" << "Stretch" << "size"<< "80" <<
                       "displayRole" << "$disp");
    dialog->setHeaderItem(headeritemsVarLst);
    dialog->setPrimaryKey("name");
    dialog->setDataKeyList(QStringList() << "name" << "disp");
    dialog->setSelectionMode(QAbstractItemView::SingleSelection);
    dialog->setSearchText("");
    dialog->searchEntry()->setPlaceholderText(ttr("Search %1"));
    dialog->searchEntry()->setSearchDelayTime(1);
    dialog->searchEntry()->setOptionList(TDataParse::headerItem2searchList(headeritemsVarLst));
    dialog->setSearchKeys(QStringList() << "name" );
    dialog->tableView()->verticalHeader()->setVisible(false);
    QString verTypeStr = APP->versionType();
    if (verTypeStr.isEmpty()) verTypeStr = "Default";
    TDataResponse dataRes = TOPDAM->dataResponse(":/topclass/GetConfigModuleList.js", verTypeStr);
    if(dataRes.hasError())
    {
        TMessageBox::error(this,
                           tr("Get module list failed!"),
                           dataRes.errText());
        return QString("");
    }
    QVariantList rowVarLst = dataRes.data().toList();
    dialog->loadData(rowVarLst);
    //显示窗口
    QVariantList selectVarLst = dialog->run();
    if(selectVarLst.isEmpty()) return QString("");
    //取值并返回
    QVariantMap selVarMap = selectVarLst.at(0).toMap();
    QString urlStr = selVarMap.value("name").toString().toUpper();

    return urlStr;
}

QString TopWin8Desktop::selectImageDialog()
{
    QStringList mimeTypeFilters;
    foreach (const QByteArray &mimeTypeName, QImageReader::supportedMimeTypes())
        mimeTypeFilters.append(mimeTypeName);
    mimeTypeFilters.sort();
    const QStringList picturesLocations = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
    QFileDialog dialog(this, tr("Open File"),
                       picturesLocations.isEmpty() ? QDir::currentPath() : picturesLocations.first());
    dialog.setAcceptMode(QFileDialog::AcceptOpen);
    dialog.setMimeTypeFilters(mimeTypeFilters);
    dialog.selectMimeTypeFilter("image/jpeg");

    QString fileStr;
    while (dialog.exec() == QDialog::Accepted)
    {
        if (dialog.selectedFiles().count() > 0) fileStr = dialog.selectedFiles().first();

        QImage image(fileStr);
        if (image.isNull())
        {
            QMessageBox::information(this, QGuiApplication::applicationDisplayName(),
                                     tr("Cannot load %1.").arg(QDir::toNativeSeparators(fileStr)));
            setWindowFilePath(QString());
            continue;
        }
        break;
    }

    return fileStr;
}

void TopWin8Desktop::editGroup()
{
    Q_D(TopWin8Desktop);
    if (!d->mRightClickedGroup) return;

    TUiloaderDialog dialog;
    QString uimStr = TFileIo::readFileString(":/topclass/GroupInfo.tui");
    QPoint point = d->mRightClickedGroup->point();
    dialog.setTitle(tr("Edit - %1").arg(d->mRightClickedGroup->title()));
    dialog.setIcon("edit2.white");
    dialog.setSelf(this);
    dialog.setScriptEngine(APP->scriptEngine());
    dialog.resize(350, 150);
    dialog.setUiStr(uimStr);
    QVariantMap categoryDataVarMap = d->mRightClickedGroup->configData();
    QString sizeStr = categoryDataVarMap.value("size").toString();

    QSize size = sizeStr2QSize(sizeStr);
    categoryDataVarMap.insert("rowSpan", QString::number(size.width()));
    categoryDataVarMap.insert("colSpan", QString::number(size.height()));

    dialog.loadValues(categoryDataVarMap);
    QVariantMap retVarMap = dialog.run(true);
    dialog.close();
    if (retVarMap.isEmpty()) return;
    sizeStr = rowColStr2SizeStr(retVarMap.value("rowSpan").toString(), retVarMap.value("colSpan").toString());
    retVarMap.insert("size", sizeStr);
    d->mRightClickedGroup->setConfigData(retVarMap);
    d->mRightClickedGroup->setPoint(point);
    d->mRightClickedGroup->doLayout();

    d->mRightClickedGroup = NULL;
}

void TopWin8Desktop::deleteGroup()
{
    Q_D(TopWin8Desktop);
    if (!d->mRightClickedGroup) return;

    if (TMessageBox::msgbox(this,
                            tr("Are you sure to delete category group %1?").arg(d->mRightClickedGroup->title()),
                            "",
                            "Question",
                            tr("Delete"),
                            QStringList() << tr("Yes") + ":Yes:Accept" << tr("No") + ":No:Reject",
                            "No") == "Yes")
    {
        if (removeOne(d->mRightClickedGroup))
        {
            // 上传数据
            onDataChanged();
            delete d->mRightClickedGroup;
        }
    }
    d->mRightClickedGroup = NULL;
}

void TopWin8Desktop::addItem()
{
    Q_D(TopWin8Desktop);
    if (!d->mRightClickedGroup) return;

    TUiloaderDialog dialog(this);
    QString uiStr = TFileIo::readFileString(":/topclass/ItemInfo.tui");
    dialog.setTitle(tr("Add module to %1").arg(d->mRightClickedGroup->title()));
    dialog.setIcon("add.white");
    dialog.setSelf(this);
    dialog.setScriptEngine(APP->scriptEngine());
    dialog.resize(450, 570);
    dialog.setUiStr(uiStr);

    QVariantMap retVarMap = dialog.run(true);
    dialog.close();

    if (retVarMap.isEmpty()) return;

    TopDesktopItemAbs *deskItem = d->mRightClickedGroup->createItem(retVarMap);
    if (deskItem)
    {
        deskItem->setParent(this);
        deskItem->setHasOpenRight(true);
        deskItem->setProperty("VISIBLE", true);
        d->mRightClickedGroup->appendOne(deskItem);
        d->mRightClickedGroup->doLayout();
    }
    d->mRightClickedGroup = NULL;
}

void TopWin8Desktop::refresh()
{
    if (!isFromDb())
    {
        doLayout();
        return;
    }
    QVariantMap sqlVarMap;
    sqlVarMap.insert("class", "win8desktop");
    sqlVarMap.insert("name", APP->currentProduct().toUpper());

    QString verTypeStr = APP->versionType();
    if (verTypeStr.isEmpty()) verTypeStr = "Default";
    sqlVarMap.insert("ver_type", verTypeStr);
    TDataResponse dataRes = TOPDAM->dataResponse(":/topclass/GetSysWin8Desktop.js", sqlVarMap);
    if (dataRes.hasError())
    {
        messageBar()->doError(ttr("Load Data failed!"), dataRes.errText(), 2000);
    }
    else
    {
        setConfigData(TDataParse::jsonStr2Variant(dataRes.data().toString()).toMap());

        doLayout();
        setDataModified(false);
        setEditable(false);
    }
}

void TopWin8Desktop::editDesktop()
{
    setEditable(true);
}

void TopWin8Desktop::saveDesktop()
{
    QVariantMap sqlVarMap;
    sqlVarMap.insert("class", "win8desktop");
    sqlVarMap.insert("name", APP->currentProduct().toUpper());
    sqlVarMap.insert("value", TDataParse::variant2JsonStr(configData()));

    QString verTypeStr = APP->versionType();
    if (verTypeStr.isEmpty()) verTypeStr = "Default";
    sqlVarMap.insert("ver_type", verTypeStr);

    QString uuidStr = TDataParse::getVariantMd5(sqlVarMap);
    sqlVarMap.insert("uuid", uuidStr);

    TDataResponse dataRes = TOPDAM->dataResponse(":/topclass/UpdateSysWin8DesktopItem.js", sqlVarMap);
    if(dataRes.hasError())
    {
        messageBar()->doError(ttr("Upload Desktop data failed!"), dataRes.errText(), 2000);
    }
    else
    {
        setEditable(false);
        setDataModified(false);
    }
}

void TopWin8Desktop::setDesktopBackground()
{
    Q_D(TopWin8Desktop);
    QString uiStr = TFileIo::readFileString(":/topclass/DesktopInfo.tui");
    TUiloaderDialog dialog;
    dialog.setTitle(tr("Edit Config"));
    dialog.setIcon("setting2.white");
    dialog.setSelf(this);
    dialog.setScriptEngine(APP->scriptEngine());
    dialog.resize(350, 100);
    dialog.setUiStr(uiStr);
    QVariantMap dataVarMap;
    dataVarMap.insert("background", config("background").toString());
    dataVarMap.insert("backgroundImage", config("backgroundImage").toString());
    dialog.loadValues(dataVarMap);
    QVariantMap retVarMap = dialog.run();
    dialog.close();
    if (retVarMap.isEmpty()) return;

//        this->setConfig("backgroundImage", "F:/SvnData/topikm3/topikm3_frame/topcore/src/topclass/desktop.JPG");
    if (this->config("background").toString() != retVarMap.value("background").toString()
            || this->config("backgroundImage").toString() != retVarMap.value("backgroundImage").toString())
    {
        this->setConfig("background", retVarMap.value("background").toString());
        this->setConfig("backgroundImage", retVarMap.value("backgroundImage").toString());
        setBackground();
        setDataModified(true);
    }


}

void TopWin8Desktop::addGroup()
{
    Q_D(TopWin8Desktop);
    TUiloaderDialog dialog;
    QString uimStr = TFileIo::readFileString(":/topclass/GroupInfo.tui");
    dialog.setTitle(tr("Add Group"));
    dialog.setIcon("add");
    dialog.setSelf(this);
    dialog.setScriptEngine(APP->scriptEngine());
    dialog.resize(350, 150);
    dialog.setUiStr(uimStr);
    QVariantMap retVarMap = dialog.run(true);
    dialog.close();

    if (retVarMap.isEmpty()) return;

    retVarMap.insert("size", QString("%1x%2").arg(retVarMap.value("rowSpan").toString())
                     .arg(retVarMap.value("colSpan").toString()));
    TopDesktopGroupAbs *deskcate = createGroup(retVarMap);

    deskcate->setPoint(classPointToCenterWidgetPoint(d->mDesktopMenuFromDb->property("POSITION").toPoint()));
    appendOne(deskcate);
    doLayout();
    onDataChanged();
}

void TopWin8Desktop::editDesktopPopupConfig()
{
    QString uiStr = TFileIo::readFileString(":/topclass/DesktopPopupInfo.tui");
    TUiloaderDialog dialog;
    dialog.setTitle(tr("Edit Config"));
    dialog.setIcon("setting2.white");
    dialog.setSelf(this);
    dialog.setScriptEngine(APP->scriptEngine());
    dialog.resize(350, 100);
    dialog.setUiStr(uiStr);
    QVariantMap dataVarMap = configData();
    dataVarMap.remove("groups");
    QSize size = sizeStr2QSize(dataVarMap.value("size").toString());
    dataVarMap.insert("rowSpan", QString::number(size.width()));
    dataVarMap.insert("colSpan", QString::number(size.height()));
    dialog.loadValues(dataVarMap);
    QVariantMap retVarMap = dialog.run();
    dialog.close();
    if (retVarMap.isEmpty()) return;

    retVarMap.insert("size", QString("%1x%2").arg(retVarMap.value("rowSpan").toString())
                     .arg(retVarMap.value("colSpan").toString()));
    retVarMap.insert("type", "group");
    retVarMap.insert("size", rowColStr2SizeStr(retVarMap.value("rowSpan").toString(),
                                               retVarMap.value("colSpan").toString()));

    setConfigData(retVarMap);
    setBackground();
    setDataModified(true);
}

void TopWin8Desktop::setBackground()
{

    qDebug() << this << QString(".TopWin8Desktop{background-color:%1;border-image:url(%2)}")
                .arg(this->config("background").toString())
                .arg(this->config("backgroundImage").toString());


    this->setStyleSheet(QString(".TopWin8Desktop{background-color:%1;border-image:url(%2)}")
                        .arg(this->config("background").toString())
                        .arg(this->config("backgroundImage").toString()));
}

TopDesktopGroupAbs *TopWin8Desktop::createGroup(const QVariant &iGroupVar)
{
    Q_D(TopWin8Desktop);
    QVariantMap groupsVarMap = iGroupVar.toMap();

    TopDesktopGroupAbs *group = new TopWin8DesktopGroup(d->mScrollCenterWgt);
    group->installEventFilter(this);
    group->setCellWidth(TTHEME_SCALE(WndSize_CellWidth));
    group->setCellHeight(TTHEME_SCALE(WndSize_CellHeight));

    QString titleStr = groupsVarMap.value(QString("title_%1").arg(APP->language())).toString();
    if (titleStr.isEmpty()) titleStr = groupsVarMap.value("title").toString();
    group->setTitle(titleStr);
    group->setConfigData(groupsVarMap);
    group->setEditable(editable());
    group->setProperty("MODULE", this->property("MODULE").toString());
    group->doLayout();

    connect(group, SIGNAL(dataChanged()), this, SLOT(onDataChanged()));
    connect(group, SIGNAL(urlRequested(QString)), this, SIGNAL(urlRequested(QString)));

    return group;
}

void TopWin8Desktop::wheelEvent(QWheelEvent *iEvent)
{
    Q_D(TopWin8Desktop);
    int deltaInt = iEvent->delta();
    QScrollBar *scrl = d->mScrollArea->horizontalScrollBar();
    scrl->setValue(scrl->value() - deltaInt);

    TopDesktopAbs::wheelEvent(iEvent);
}

void TopWin8Desktop::dragEnterEvent(QDragEnterEvent *iEvent)
{
    if (iEvent->mimeData()->hasFormat("application/desktop-groupdata"))
    {
        iEvent->setDropAction(Qt::MoveAction);
        iEvent->accept();
    }
    else
    {
        iEvent->ignore();
    }
}

void TopWin8Desktop::dropEvent(QDropEvent *iEvent)
{
    if (iEvent->mimeData()->hasFormat("application/desktop-groupdata"))
    {
        QByteArray categoryDataBytLst = iEvent->mimeData()->data("application/desktop-groupdata");
        QDataStream dataStream(&categoryDataBytLst, QIODevice::ReadOnly);

        QPoint offsetPos;
        QString titleStr;
        QSize size, cellSize;

        QString categoryStr;
        QVariantMap categoryDataVarMap;
        dataStream >> offsetPos >> titleStr >> size >> cellSize >> categoryStr;
        categoryDataVarMap = TDataParse::jsonStr2Variant(categoryStr).toMap();

        TopDesktopGroupAbs *newCate = createGroup(categoryDataVarMap);
        // 关闭自动删除
        newCate->setAttribute(Qt::WA_DeleteOnClose);

        QPoint atWgtPoint = classPointToCenterWidgetPoint(iEvent->pos() - offsetPos);
        if (atWgtPoint.x() < 0) atWgtPoint.setX(0);
        if (atWgtPoint.y() < 0) atWgtPoint.setY(0);
        newCate->setPoint(atWgtPoint);
        removeOne(qobject_cast<TopDesktopGroupAbs *>(iEvent->source()));

        // 在最后触发
        if (iEvent->source()->property("MODULE").toString() == this->property("MODULE").toString())
        {
            iEvent->setDropAction(Qt::MoveAction);
            iEvent->accept();
        }
        else
        {
            iEvent->setDropAction(Qt::CopyAction);
        }

        appendOne(newCate);
        doLayout();

        onDataChanged();
    }
}

bool TopWin8Desktop::eventFilter(QObject *iObj, QEvent *iEvent)
{
    Q_D(TopWin8Desktop);
    if (iEvent->type() == QEvent::MouseButtonPress
            && iObj->metaObject()->className() == TopWin8DesktopGroup::staticMetaObject.className())
    {
        QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(iEvent);
        if (mouseEvent->button() == Qt::RightButton)
        {
            d->mRightClickedGroup = qobject_cast<TopWin8DesktopGroup *>(iObj);
            if (d->mGroupMenu && editable())
            {
                d->mGroupMenu->setProperty("POSITION", mouseEvent->pos());
                d->mGroupMenu->exec(QCursor::pos());
            }
            return true;
        }

    }
    else if (iEvent->type() == QEvent::MouseButtonPress
             && iObj != this)
    {
        QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(iEvent);
        if (mouseEvent && mouseEvent->button() == Qt::RightButton)
        {
            return true;
        }
    }
    else if (iEvent->type() == QEvent::MouseButtonPress && iObj == this)
    {
        QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(iEvent);
        if (mouseEvent->button() == Qt::RightButton)
        {
            if (isFromDb() && d->mDesktopMenuFromDb)
            {
                d->mDesktopActionsFromDb.value(DesktopAction_SetEditable)->setDisabled(editable() && isDataModified());
                d->mDesktopActionsFromDb.value(DesktopAction_SaveModified)->setDisabled(!isDataModified());
                d->mDesktopActionsFromDb.value(DesktopAction_SetBackground)->setDisabled(!editable());
                d->mDesktopActionsFromDb.value(DesktopAction_AddGroup)->setDisabled(!editable());
                d->mDesktopMenuFromDb->setProperty("POSITION", mouseEvent->pos());
                d->mDesktopMenuFromDb->exec(QCursor::pos());
                return true;
            }
            else if (!isFromDb() && d->mDesktopMenu)
            {
                d->mDesktopMenu->exec(QCursor::pos());
                return true;
            }
        }
    }
    return TopDesktopAbs::eventFilter(iObj, iEvent);
}

void TopWin8Desktop::paintEvent(QPaintEvent *iEvent)
{
    if (!isFromDb() && editable())
    {
        QPainter painter(this);
        QColor color(QColor(TTHEME->value("TITLE")));
        QBrush brush(color);
        QPen pen(brush, 1, Qt::DashDotLine, Qt::RoundCap, Qt::RoundJoin);
        painter.setPen(pen);
        QSize size = sizeStr2QSize(config("size").toString());
        painter.drawLine(QPoint(0, size.height()), QPoint(size.width(), size.height()));
        painter.drawLine(QPoint(size.width(), 0), QPoint(size.width(), size.height()));
    }
    TopDesktopAbs::paintEvent(iEvent);
}

void TopWin8Desktop::mouseMoveEvent(QMouseEvent *iEvent)
{
    if (!isFromDb() && editable())
    {
        QRect rect(QPoint(0,0), QSize(sizeStr2QSize(config("size").toString())));
        if (rect.contains(iEvent->pos()))
        {
            QToolTip::showText(iEvent->globalPos(), tr("Size(%1)").arg(config("size").toString()), this);
            this->setToolTipDuration(20);
        }
    }
}

QPoint TopWin8Desktop::classPointToCenterWidgetPoint(QPoint iPoint)
{
    Q_D(TopWin8Desktop);
    return iPoint - d->mScrollArea->pos() - d->mScrollCenterWgt->pos();
}

void TopWin8Desktop:: loadLanguagePkg()
{
    // item TUI语言包
    insertLanguageMap("Plugin Name", tr("Plugin Name"));
    insertLanguageMap("Class Name", tr("Class Name"));
    insertLanguageMap("Title", tr("Title"));
    insertLanguageMap("Title(English)", tr("Title(English)"));
    insertLanguageMap("Title(Chinese Simplified)", tr("Title(Chinese Simplified)"));
    insertLanguageMap("Title(Chinese Tranditional)", tr("Title(Chinese Tranditional)"));
    insertLanguageMap("Title(Japanese)", tr("Title(Japanese)"));
    insertLanguageMap("Title(Ot)", tr("Title(Ot)"));
    insertLanguageMap("Icon Name", tr("Icon Name"));
    insertLanguageMap("Remark", tr("Remark"));
    insertLanguageMap("Remark(English)", tr("Remark(English)"));
    insertLanguageMap("Remark(Chinese Simplified)", tr("Remark(Chinese Simplified)"));
    insertLanguageMap("Remark(Chinese Tranditional)", tr("Remark(Chinese Tranditional)"));
    insertLanguageMap("Remark(Japanese)", tr("Remark(Japanese)"));
    insertLanguageMap("Remark(Ot)", tr("Remark(Ot)"));
    insertLanguageMap("Layout Size", tr("Layout Size"));
    insertLanguageMap("1x1", tr("1x1"));
    insertLanguageMap("1x2", tr("1x2"));
    insertLanguageMap("2x1", tr("2x1"));
    insertLanguageMap("2x2", tr("2x2"));
    insertLanguageMap("Icon Size", tr("Icon Size"));
    insertLanguageMap("Background Color", tr("Background Color"));
    insertLanguageMap("Url Address", tr("Url Address"));
    insertLanguageMap("Search", tr("Search"));
    insertLanguageMap("Module Name", tr("Module Name"));
    insertLanguageMap("Search %1", tr("Search %1"));

    //分组tui语言包
    insertLanguageMap("GroupName", tr("GroupName"));
    insertLanguageMap("GroupName(English)", tr("GroupName(English)"));
    insertLanguageMap("GroupName(Chinese Simplified)", tr("GroupName(Chinese Simplified)"));
    insertLanguageMap("GroupName(Chinese Tranditional)", tr("GroupName(Chinese Tranditional)"));
    insertLanguageMap("GroupName(Japanese)", tr("GroupName(Japanese)"));
    insertLanguageMap("GroupName(Ot)", tr("GroupName(Ot)"));
    insertLanguageMap("Icon Name", tr("Icon Name"));
    insertLanguageMap("Background Color", tr("Background Color"));
    insertLanguageMap("Border Color", tr("Border Color"));
    insertLanguageMap("Row Span", tr("Row Span"));
    insertLanguageMap("Col Span", tr("Col Span"));

    // 桌面
    insertLanguageMap("Background Color", tr("Background Color"));
    insertLanguageMap("Background Image", tr("Background Image"));

    insertLanguageMap("Popup Position", tr("Popup Position"));
    insertLanguageMap("Menu", tr("Menu"));
    insertLanguageMap("Center Area", tr("Center Area"));
    insertLanguageMap("Left Area", tr("Left Area"));
    insertLanguageMap("Top Area", tr("Top Area"));
    insertLanguageMap("Right Area", tr("Right Area"));
    insertLanguageMap("Bottom Area", tr("Bottom Area"));
    insertLanguageMap("Row Span(px)", tr("Row Span(px)"));
    insertLanguageMap("Col Span(px)", tr("Col Span(px)"));
}

void TopWin8Desktop::insertLanguageMap(const QString &iKeyStr, const QString &iValueStr)
{
    Q_D(TopWin8Desktop);
    d->mLanguageMap.insert(iKeyStr, iValueStr);
}


