﻿#include "sysusermgtthread.h"
#include <topcore/topcore.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tdataparse.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>

SysUserMgtThread::SysUserMgtThread(QObject *iParent)
    : TopClassThreadAbs(iParent)
{

}

SysUserMgtThread::~SysUserMgtThread()
{

}

void SysUserMgtThread::run()
{
    if (invokeName() == "DELETE_USER") {
        deleteUser();
    } else if (invokeName() == "RESET_PASSWORD") {
        resetPassword(invokeParameter().toMap());
    } else if (invokeName() == "RESET_PASSWORD_VALIDITY_DATE") {
        resetPasswordValidityDate(invokeParameter().toMap());
    }
}

void SysUserMgtThread::deleteUser()
{
    TDataResponse dataRes;

    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        TSqlDeleterV2 sqlDeleter;
        sqlDeleter.setTable("pub_contacts");
        sqlDeleter.setWhere(QString("id IN (SELECT contcat_id FROM sys_user WHERE id IN (%1) AND contcat_id IS NOT NULL)").arg(invokeParameter().toStringList().join(",")));
        sqlQuery.deleteRow(sqlDeleter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlDeleter.clear();
        sqlDeleter.setTable("sys_user");
        sqlDeleter.setWhere("id", invokeParameter());
        sqlQuery.deleteRow(sqlDeleter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlDeleter.clear();
        sqlDeleter.setTable("sys_role_map_user");
        sqlDeleter.setWhere("user_id", invokeParameter());
        sqlQuery.deleteRow(sqlDeleter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlQuery.commit();
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}

void SysUserMgtThread::resetPassword(const QVariantMap &iDataMap)
{
    TDataResponse dataRes;

    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        QString password = iDataMap.value("password").toString();
        if (password.isEmpty()) {
            password = QStringLiteral("123456");
        }
        QVariantMap updateData;
        updateData.insert("password", TDataParse::getVariantMd5(password));
        TSqlUpdaterV2 updater;
        updater.setTable("sys_user");
        updater.setWhere("username", iDataMap.value("username"));
        updater.setData(updateData);
        sqlQuery.updateRow(updater);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlQuery.commit();
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}

void SysUserMgtThread::resetPasswordValidityDate(const QVariantMap &iDataMap)
{
    TDataResponse dataRes;

    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        QVariantMap updateData;
        QVariantMap attrData;
        attrData.insert("effective_date", APP->getServerToday());
        updateData.insert("attr_data", attrData);
        TSqlUpdaterV2 updater;
        updater.setTable("sys_user");
        updater.setWhere("id", iDataMap.value("id"));
        updater.setData(updateData);
        updater.setUpdatePolicy("attr_data", "json_merge");
        sqlQuery.updateRow(updater);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlQuery.commit();
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}
